/*---------------------------------------------------------------------------------------------------*\
|   PROGRAM NAME: rx_lookup_keys.sas                                                                  |
|    Adds two surrogate keys to the NDC_Lookup_Table: DrugClassKey and GenericNameKey                 |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Version: Revised 1.0                                                                              |
|   Date: August 27, 2015                                                                             |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|       NDC_Lookup_Table                                                                              |
|                                                                                                     |
|   Program outputs:                                                                                  |
|       NDC_Lookup_Table_Keyed                                                                        |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
\*---------------------------------------------------------------------------------------------------*/

*** 10/7/15 - MR (1) Remove known garbage upfront to reduce the number of DrugClasses per NDC ;
***                  and reduce (useless) multiplication of NDC claims when joining to dispensings.;
***                  This small change cuts the number of NDC that have multple DrugClasses in half.;
***              (2) Save key macro variables into a metadata table so that definitions may be retrieved;
***                  as needed by other modules without having to run this module again first;
*** 10/08/15 -MR (1) Read initial lookup from INFOLDER and output results to DPTEMP;

*** 10/11/15 -RR (1) Reset original code for initial SORT that removes duplicates;
***              (2) Moved code that saved key macro vars into a metadata table, to very end of program;

*** 10/20/15 -RR (1) Set exclusions to include blank descriptions;
*** 10/23/15 -MR (1) Replace NumLength function with NumLength macro due to unresolved %sysfunc issues;
*** 12/4/15  -RR (1) Ensured that duplicate keys for DrugClass and GenericName are written out only once
                     for the flattened table;

%macro RxLookupKeys;
  %*** Set local macro variables;
  %local NDC_lookup_table_msoc ExcludedValues DrugClassCount GenericNameCount DrugClassLength GenericNameLength MaxDrugClass MaxGenericName
    Ct_DrugClass_Mult Ct_GenericName_Mult;

  %let NDCTbl=NDC_lookup_table;

  %*** MOD 10/7/15: Create list of values to be excluded in UPPERCASE;
  %let ExcludedValues=UNKNOWN;

  * Remove "pure" duplicates from lookup table;
  proc sort data=infolder.&NDCTbl.(keep=NDC DrugClass GenericName) out=&NDCTbl._uni nodupkey;
    by NDC DrugClass GenericName;
  run;

  **** Determine List and # of keys needed for each of Drug Class and Generic Name;
  * Read lookup table once, creating 2 output datasets by Drug Class and Generic Name, excluding bad values;
  proc summary data=&NDCTbl._uni;
    class DrugClass GenericName ;
    output out=DrugClass  (keep=_type_ DrugClass   where=(_type_=2 and DrugClass NE " "   and upcase(DrugClass)   not in (%quotelst(&ExcludedValues.))));
    output out=GenericName(keep=_type_ GenericName where=(_type_=1 and GenericName NE " " and upcase(GenericName) not in (%quotelst(&ExcludedValues.))));
    types DrugClass GenericName ;
  run;

  * Determine # of values in each dataset;
  proc sql noprint;
    select count(*) into :DrugClassCount
    from DrugClass
    ;
    select count(*) into :GenericNameCount
    from GenericName
    ;
  quit;
  %if &debug.=Y %then %do;
    %put # of keys: &=DrugClassCount. and &=GenericNameCount.;
  %end;

  * Set numeric length of key, based on # of values;
  %let DrugClassLength=  %NumLength(&DrugClassCount.);
  %let GenericNameLength=%NumLength(&GenericNameCount.);
  %put &=DrugClassLength. and &=GenericNameLength.;

  *** Create keys and final lookup tables;
  * Drug Class;
  data DPtemp.DrugClassLookup ;
    length DrugClassKey &DrugClassLength.;
    set DrugClass(keep=DrugClass);
    DrugClassKey=_N_;
  run;

  * Generic Name;
  data DPtemp.GenericNameLookup ;
    length GenericNameKey &GenericNameLength.;
    set GenericName(keep=GenericName);
    GenericNameKey=_N_;
  run;

  proc sql _method;
    *** Combine keys with lookup table and output;
    create table DPtemp.&NDCTbl._Keyed as
    select base.NDC, base.GenericName, base.DrugClass
      , DC.DrugClassKey
      , GN.GenericNameKey
    from &NDCTbl._uni as base
    left join DPtemp.DrugClassLookup as DC
      on base.DrugClass = DC.DrugClass
      left join DPtemp.GenericNameLookup as GN
        on base.GenericName = GN.GenericName
    order by NDC, GenericName, DrugClass
    ;
  quit;

  *********************************************** BEGIN: For use with Hash Lookup - Flattened lookup table *********************************************** ;
  proc sql noprint;
    ** Get maximum number of possible DrugClass keys and GenericName keys per NDC;
    select put(max(DrugClasses),1.)
      , put(max(GenericNames),1.) into :MaxDrugClass, :MaxGenericName
    from
      (select count(distinct DrugClass) as DrugClasses
        , count(distinct GenericName)  as GenericNames
      from DPtemp.&NDCTbl._Keyed
      group by NDC)
    ;
  quit;
  %if &debug.=Y %then %do;
    %put Maximum # of keys: &=MaxDrugClass. and &=MaxGenericName.;
  %end;

  %**** MOD 12/4/15: Calculate lengths for comparison string variables;
  %let DCCompareLength=%eval(&MaxDrugClass. * (%length(&DrugClassCount.)+1));
  %let GNCompareLength=%eval(&MaxGenericName. * (%length(&GenericNameCount.)+1));
  %if &debug.=Y %then %do;
    %put Size of compare strings needed: &=DCCompareLength. and &=GNCompareLength.;
  %end;

  ** Flatten table;
  data DPTemp.NDC_Lookup_Flat(keep=NDC DrugClassKey1-DrugClassKey&MaxDrugClass. GenericNameKey1-GenericNameKey&MaxGenericName.);
    length DrugClassKey1-DrugClassKey&MaxDrugClass. &DrugClassLength. GenericNameKey1-GenericNameKey&MaxGenericName. &GenericNameLength.
      _DCValues $&DCCompareLength. _GNValues $&GNCompareLength.;
    retain _DC_idx _GN_idx _DCValues _GNValues DrugClassKey1-DrugClassKey&MaxDrugClass. GenericNameKey1-GenericNameKey&MaxGenericName.;
    set DPtemp.&NDCTbl._Keyed;
    by NDC;
    ** Set arrays to the size needed for each of DrugClass and GenericName key variables;
    array DC [*] DrugClassKey1-DrugClassKey&MaxDrugClass.;
    array GN [*] GenericNameKey1-GenericNameKey&MaxGenericName.;
    if first.NDC then do;
      ** Clean arrays;
      do _i=1 to dim(DC);
        DC[_i]=.;
      end;
      do _i=1 to dim(GN);
        GN[_i]=.;
      end;
      ** Initialize index values into the arrays;
      _DC_idx=0;
      _GN_idx=0;
      ** Initialize string compares;
      _DCValues="";
      _GNValues="";
    end;
    ** For each of DrugClass and GenericName, test current to all prior rows within NDC
    ** If not equal to other rows, then:
    ** - Increment index values
    ** - Fill the array variable
    ** - Set new comparison values;
    if not indexw(_DCValues,strip(put(DrugClassKey,4.))) and DrugClassKey NE . then do;  %** MOD 12/4/15;
      _DC_idx + 1;
      DC[_DC_idx]=DrugClassKey;
      _DCValues=catx(" ",_DCValues,strip(put(DrugClassKey,4.)));
    end;
    if not indexw(_GNValues,strip(put(GenericNameKey,4.))) and GenericNameKey NE . then do; %** MOD 12/4/15;
      _GN_idx + 1;
      GN[_GN_idx]=GenericNameKey;
      _GNValues=catx(" ",_GNValues,strip(put(GenericNameKey,4.)));
    end;
    ** Output only once per NDC;
    if last.NDC then do;
      output;
    end;
  run;

  *********************************************** END: For use with Hash Lookup - Flattened lookup table *********************************************** ;

  *** All of next processing is for MSOC only;
  %if %upcase(&DPID.)=MS %then %do;
    proc sql noprint;
      *** Create list of distinct DrugClasses that share NDCs;
      create table DrugClassShareNDC as
      select distinct DrugClass from
        (select NDC, DrugClass, count(distinct DrugClass) as DrugClasses
        from DPtemp.&NDCTbl._Keyed
        where DrugClassKey NE .
        group by NDC)
      where DrugClasses > 1
      order by DrugClass
      ;
      *** Get count of DrugClasses that share NDCs;
      select strip(put(count(distinct DrugClass),5.)) into :Ct_DrugClass_Mult
      from DrugClassShareNDC
      ;
      *** Create list of distinct GenericNames that share NDCs;
      create table GenericNameShareNDC as
      select distinct GenericName from
        (select NDC, GenericName, count(distinct GenericName) as GenericNames
        from DPtemp.&NDCTbl._Keyed
        where GenericNameKey NE .
        group by NDC)
      where GenericNames > 1
      order by GenericName
      ;
      *** Get count of GenericNames that share NDCs;
      select strip(put(count(distinct GenericName),5.)) into :Ct_GenericName_Mult
      from GenericNameShareNDC
      ;
    quit;
    %put There are %trim(&Ct_DrugClass_Mult.) Drug Classes that share NDCs;
    %put There are %trim(&Ct_GenericName_Mult.) Generic Names that share NDCs;

    ********** Output report of Drug Classes and Generic Names that share NDCs to PDF report. For MSOC only **********;
    *** Close LISTING location, set PDF location, output file and open the file;
    ods listing close;
    ods pdf file="&MSOC.ShareNDCs.pdf";

    title1 "Drug Classes that Share NDCs: %trim(&Ct_DrugClass_Mult.) of %trim(&DrugClassCount.) (%sysfunc(round(%sysevalf(&Ct_DrugClass_Mult.*100/&DrugClassCount.),.1))%) total";
    proc print data=DrugClassShareNDC label;
      var DrugClass;
      label DrugClass="Drug Class";
    run; title1;

    title1 "Generic Names that Share NDCs: %trim(&Ct_GenericName_Mult.) of %trim(&GenericNameCount.) (%sysfunc(round(%sysevalf(&Ct_GenericName_Mult.*100/&GenericNameCount.),.1))%) total";
    proc print data=GenericNameShareNDC label;
      var GenericName;
      label GenericName="Generic Name";
    run; title1;

    *** Close PDF file and open LISTING location;
    ods pdf close;
    ods listing;
  %end;

  ** Save macro variables into DPtemp.MaxDrugsMeta;
  ** so that RX_ATTACH_KEYS module may retrieve them;
  data DPtemp.MaxDrugsMeta;
    length name $32. value $2.;
    name= "MaxDrugClass"; value= "&MaxDrugClass."; output;
    name= "MaxGenericName"; value= "&MaxGenericName."; output;
  run;
%mend;
%RxLookupKeys;


* End of rx_lookup_keys.sas;
